"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("@apollo/client");
// Optimistic response generation utility method
//
function generateOptimisticResponseForMutation({ operationType, entityName, objects }) {
    const optimisticResponse = {
        __typename: "mutation_root",
        [`${operationType}_${entityName}`]: {
            affected_rows: objects.length,
            returning: objects.map(o => {
                if (operationType === "insert")
                    return convertInsertInputToPartialFragmentResursive({ insertInput: o });
                else
                    return { ...o, __typename: entityName };
            }),
            __typename: `${entityName}_mutation_response`
        }
    };
    return optimisticResponse;
}
exports.generateOptimisticResponseForMutation = generateOptimisticResponseForMutation;
/**
 *
 * @param param0
 */
function generateUpdateFunctionForMutation({ operationType, entityName, entityId }) {
    return (cache, result) => {
        cache.evict(client_1.defaultDataIdFromObject({ __typename: entityName, id: entityId }));
    };
}
exports.generateUpdateFunctionForMutation = generateUpdateFunctionForMutation;
function IS_JAVASCRIPT_SCALAR_EQUIVALENT(arg, includeNull = false) {
    var type = typeof arg;
    return (includeNull && arg === null) || (type != "object" && type != "function");
}
exports.IS_JAVASCRIPT_SCALAR_EQUIVALENT = IS_JAVASCRIPT_SCALAR_EQUIVALENT;
function IS_APOLLO_OBJECT(o) {
    return o && o.id && o.__typename;
}
exports.IS_APOLLO_OBJECT = IS_APOLLO_OBJECT;
function IS_NON_NULL_OBJECT(o) {
    return typeof o === "object" && o !== null;
}
exports.IS_NON_NULL_OBJECT = IS_NON_NULL_OBJECT;
function IS_OBJECT_WITH_ID(o) {
    return IS_NON_NULL_OBJECT(o) && o.id;
}
exports.IS_OBJECT_WITH_ID = IS_OBJECT_WITH_ID;
function IS_INSERT_INPUT_OBJECT(o) {
    if (!IS_NON_NULL_OBJECT(o))
        return false;
    return o.data;
}
exports.IS_INSERT_INPUT_OBJECT = IS_INSERT_INPUT_OBJECT;
function GET_INSERT_INPUT_DATA(o) {
    if (!IS_NON_NULL_OBJECT(o))
        throw new Error(`Provided object was not of type InsertInput (with data property): ${JSON.stringify(o)}`);
    return o.data;
}
exports.GET_INSERT_INPUT_DATA = GET_INSERT_INPUT_DATA;
function convertApolloObjectToRefObj(o) {
    if (!IS_APOLLO_OBJECT(o))
        throw new Error(`Provided object was not of type ApolloObject (with id & _typename properties): ${JSON.stringify(o)}`);
    return { id: o.id, __typename: o.__typeName };
}
exports.convertApolloObjectToRefObj = convertApolloObjectToRefObj;
function convertApolloObjectArrayToRefObj(o) {
    return o.map(arrayItem => convertApolloObjectToRefObj(o));
}
exports.convertApolloObjectArrayToRefObj = convertApolloObjectArrayToRefObj;
function ensureTypenameOnFragment(o, typename) {
    if (!IS_OBJECT_WITH_ID(o))
        throw new Error(`Provided object was not of type ObjWithId: ${JSON.stringify(o)}`);
    return { ...o, __typename: typename };
}
exports.ensureTypenameOnFragment = ensureTypenameOnFragment;
function ensureTypenameOnFragments(o, typename) {
    return o.map(arrayItem => ensureTypenameOnFragment(o, typename));
}
exports.ensureTypenameOnFragments = ensureTypenameOnFragments;
//
// Series of functions for helping handle translations between input objects and fragments
//
function addTypenameToObjWithId({ o, typename }) {
    if (!IS_OBJECT_WITH_ID(o))
        throw new Error(`Provided object was not of type ObjWithId: ${JSON.stringify(o)}`);
    return { ...o, __typename: typename };
}
exports.addTypenameToObjWithId = addTypenameToObjWithId;
/**
 *
 */
function addTypenameToObjWithIdArray({ o, typename }) {
    return o.map(arrayItem => addTypenameToObjWithId({ o: arrayItem, typename }));
}
exports.addTypenameToObjWithIdArray = addTypenameToObjWithIdArray;
//
// Series of functions for helping handle translations between input objects and fragments
//
function convertObjWithIdToFragmentR({ o, propertyKey, fieldMap }) {
    if (!IS_OBJECT_WITH_ID(o))
        throw new Error(`Provided object was not of type ObjWithId: ${JSON.stringify(o)}`);
    let convertedObject = convertInsertInputToPartialFragmentResursive({ insertInput: o, fieldMap });
    if (fieldMap && fieldMap[propertyKey])
        convertedObject = addTypenameToObjWithId({ o: convertedObject, typename: fieldMap[propertyKey] });
    return convertedObject;
}
/**
 *
 */
function convertObjWithIdArrayToFragmentArrayR({ o, propertyKey, fieldMap }) {
    return o.map(arrayItem => convertObjWithIdToFragmentR({ o: arrayItem, propertyKey, fieldMap }));
}
/**
 *
 */
function convertObjectToFragmentR({ o, propertyKey, fieldMap }) {
    if (!IS_NON_NULL_OBJECT(o))
        return null;
    if (Array.isArray(o)) {
        if (o.length === 0 || !o[0] || !IS_OBJECT_WITH_ID(o[0]))
            return null;
        return convertObjWithIdArrayToFragmentArrayR({ o, propertyKey, fieldMap });
    }
    if (IS_OBJECT_WITH_ID(o)) {
        return convertObjWithIdToFragmentR({ o, propertyKey, fieldMap });
    }
    if (IS_INSERT_INPUT_OBJECT(o) && fieldMap[propertyKey]) {
        return convertObjectToFragmentR({ o: o.data, propertyKey, fieldMap });
    }
    // if non of the above, return null
    return null;
}
/*
 *
 */
function convertInsertInputToPartialFragmentResursive({ insertInput, fieldMap }) {
    const fragment = {};
    //Loop object and build up a fragment appropriate for a cache-add
    for (const [insertInputKey, insertInputValue] of Object.entries(insertInput)) {
        //Add scalar values
        if (IS_JAVASCRIPT_SCALAR_EQUIVALENT(insertInputValue)) {
            fragment[insertInputKey] = insertInputValue;
            continue;
        }
        // if non-null object AND no flag to explicity ignore field
        if (IS_NON_NULL_OBJECT(insertInputValue) && (!fieldMap || !fieldMap[insertInputKey] || fieldMap[insertInputKey] !== "IGNORE_FIELD")) {
            const ref = convertObjectToFragmentR({ o: insertInputValue, propertyKey: insertInputKey, fieldMap });
            if (ref) {
                fragment[insertInputKey] = ref;
                continue;
            }
        }
    }
    return fragment;
}
exports.convertInsertInputToPartialFragmentResursive = convertInsertInputToPartialFragmentResursive;
//# sourceMappingURL=utils.js.map